@@ -1,31 +1,26 @@
 #include "config.h"
 
-#include <iterator>
-#include <stdlib.h>
-#include <rak/file_stat.h>
+#include "download/download_wrapper.h"
 
 #include "data/chunk_list.h"
 #include "data/hash_queue.h"
 #include "data/hash_torrent.h"
+#include "download/available_list.h"
+#include "download/chunk_selector.h"
 #include "protocol/handshake_manager.h"
 #include "protocol/peer_connection_base.h"
-#include "torrent/exceptions.h"
-#include "torrent/object.h"
-#include "torrent/tracker_list.h"
 #include "torrent/data/file.h"
 #include "torrent/data/file_list.h"
 #include "torrent/data/file_manager.h"
-#include "torrent/peer/peer.h"
 #include "torrent/peer/connection_list.h"
-#include "torrent/tracker_controller.h"
-#include "torrent/tracker_list.h"
+#include "torrent/peer/peer.h"
+#include "torrent/tracker/manager.h"
 #include "torrent/utils/log.h"
+#include "tracker/thread_tracker.h"
+#include "tracker/tracker_controller.h"
+#include "tracker/tracker_list.h"
 #include "utils/functional.h"
-
-#include "available_list.h"
-#include "chunk_selector.h"
-
-#include "download_wrapper.h"
+#include "utils/sha1.h"
 
 #define LT_LOG_THIS(log_fmt, ...)                                       \
   lt_log_print_info(LOG_TORRENT_INFO, this->info(), "download", log_fmt, __VA_ARGS__);
@@ -34,23 +29,17 @@
 
 namespace torrent {
 
-DownloadWrapper::DownloadWrapper() :
-  m_main(new DownloadMain),
+DownloadWrapper::DownloadWrapper()
+  : m_main(new DownloadMain) {
 
-  m_bencode(NULL),
-  m_hashChecker(NULL),
-  m_connectionType(0) {
-
-  m_main->delay_download_done().slot()       = std::bind(&download_data::call_download_done, data());
-  m_main->delay_partially_done().slot()      = std::bind(&download_data::call_partially_done, data());
-  m_main->delay_partially_restarted().slot() = std::bind(&download_data::call_partially_restarted, data());
+  m_main->delay_download_done().slot()       = [this] { data()->call_download_done(); };
+  m_main->delay_partially_done().slot()      = [this] { data()->call_partially_done(); };
+  m_main->delay_partially_restarted().slot() = [this] { data()->call_partially_restarted(); };
 
   m_main->peer_list()->set_info(info());
   m_main->tracker_list()->set_info(info());
-  m_main->tracker_controller()->slot_success() = std::bind(&DownloadWrapper::receive_tracker_success, this, std::placeholders::_1);
-  m_main->tracker_controller()->slot_failure() = std::bind(&DownloadWrapper::receive_tracker_failed, this, std::placeholders::_1);
 
-  m_main->chunk_list()->slot_storage_error() = std::bind(&DownloadWrapper::receive_storage_error, this, std::placeholders::_1);
+  m_main->chunk_list()->slot_storage_error() = [this](const auto& str) { receive_storage_error(str); };
 }
 
 DownloadWrapper::~DownloadWrapper() {
@@ -60,17 +49,22 @@ DownloadWrapper::~DownloadWrapper() {
   if (info()->is_open())
     close();
 
-  // If the client wants to do a quick cleanup after calling close, it
-  // will need to manually cancel the tracker requests.
-  m_main->tracker_controller()->close();
+  // Check if needed.
+  m_main->connection_list()->clear();
+  m_main->tracker_list()->clear();
 
-  delete m_hashChecker;
-  delete m_bencode;
-  delete m_main;
+  if (m_main->tracker_controller().is_valid()) {
+    // If the client wants to do a quick cleanup after calling close, it
+    // will need to manually cancel the tracker requests.
+    m_main->tracker_controller().close();
+
+    thread_tracker()->tracker_manager()->remove_controller(m_main->tracker_controller());
+    m_main->tracker_controller().get_shared().reset();
+  }
 }
 
 void
-DownloadWrapper::initialize(const std::string& hash, const std::string& id) {
+DownloadWrapper::initialize(const std::string& hash, const std::string& id, uint32_t tracker_key) {
   char hashObfuscated[20];
   sha1_salt("req2", 4, hash.c_str(), hash.length(), hashObfuscated);
 
@@ -79,28 +73,35 @@ DownloadWrapper::initialize(const std::string& hash, const std::string& id) {
 
   info()->mutable_local_id().assign(id.c_str());
 
-  info()->slot_left()      = std::bind(&FileList::left_bytes, m_main->file_list());
-  info()->slot_completed() = std::bind(&FileList::completed_bytes, m_main->file_list());
+  info()->slot_left()      = [this] { return m_main->file_list()->left_bytes(); };
+  info()->slot_completed() = [this] { return m_main->file_list()->completed_bytes(); };
 
   file_list()->mutable_data()->mutable_hash().assign(hash.c_str());
 
-  m_main->slot_hash_check_add([this](torrent::ChunkHandle handle) { return check_chunk_hash(handle); });
+  m_main->tracker_list()->set_key(tracker_key);
+
+  m_main->slot_hash_check_add([this](torrent::ChunkHandle handle) { return check_chunk_hash(handle, false); });
 
   // Info hash must be calculate from here on.
-  m_hashChecker = new HashTorrent(m_main->chunk_list());
+  m_hash_checker = std::make_unique<HashTorrent>(m_main->chunk_list());
 
   // Connect various signals and slots.
-  m_hashChecker->slot_check_chunk() = std::bind(&DownloadWrapper::check_chunk_hash, this, std::placeholders::_1);
-  m_hashChecker->delay_checked().slot() = std::bind(&DownloadWrapper::receive_initial_hash, this);
+  m_hash_checker->slot_check_chunk()     = [this](auto h) { check_chunk_hash(h, true); };
+  m_hash_checker->delay_checked().slot() = [this] { receive_initial_hash(); };
+
+  m_main->post_initialize();
+
+  m_main->tracker_controller().set_slots([this](auto l) { return receive_tracker_success(l); },
+                                         [this](auto& m) { return receive_tracker_failed(m); });
 }
 
 void
 DownloadWrapper::close() {
   // Stop the hashing first as we need to make sure all chunks are
   // released when DownloadMain::close() is called.
-  m_hashChecker->clear();
+  m_hash_checker->clear();
 
-  // Clear after m_hashChecker to ensure that the empty hash done signal does
+  // Clear after m_hash_checker to ensure that the empty hash done signal does
   // not get passed to HashTorrent.
   hash_queue()->remove(data());
 
@@ -112,14 +113,14 @@ DownloadWrapper::close() {
   m_main->close();
 
   // Should this perhaps be in stop?
-  priority_queue_erase(&taskScheduler, &m_main->delay_download_done());
-  priority_queue_erase(&taskScheduler, &m_main->delay_partially_done());
-  priority_queue_erase(&taskScheduler, &m_main->delay_partially_restarted());
+  this_thread::scheduler()->erase(&m_main->delay_download_done());
+  this_thread::scheduler()->erase(&m_main->delay_partially_done());
+  this_thread::scheduler()->erase(&m_main->delay_partially_restarted());
 }
 
 bool
 DownloadWrapper::is_stopped() const {
-  return !m_main->tracker_controller()->is_active() && !m_main->tracker_list()->has_active();
+  return !m_main->tracker_controller().is_active() && !m_main->tracker_list()->has_active();
 }
 
 void
@@ -127,11 +128,11 @@ DownloadWrapper::receive_initial_hash() {
   if (info()->is_active())
     throw internal_error("DownloadWrapper::receive_initial_hash() but we're in a bad state.");
 
-  if (!m_hashChecker->is_checking()) {
-    receive_storage_error("Hash checker was unable to map chunk: " + std::string(rak::error_number(m_hashChecker->error_number()).c_str()));
+  if (!m_hash_checker->is_checking()) {
+    receive_storage_error("Hash checker was unable to map chunk: " + std::string(rak::error_number(m_hash_checker->error_number()).c_str()));
 
   } else {
-    m_hashChecker->confirm_checked();
+    m_hash_checker->confirm_checked();
 
     if (hash_queue()->has(data()))
       throw internal_error("DownloadWrapper::receive_initial_hash() found a chunk in the HashQueue.");
@@ -139,12 +140,13 @@ DownloadWrapper::receive_initial_hash() {
     // Initialize the ChunkSelector here so that no chunks will be
     // marked by HashTorrent that are not accounted for.
     m_main->chunk_selector()->initialize(m_main->chunk_statistics());
+
     receive_update_priorities();
   }
 
   if (data()->slot_initial_hash())
     data()->slot_initial_hash()();
-}    
+}
 
 void
 DownloadWrapper::receive_hash_done(ChunkHandle handle, const char* hash) {
@@ -154,25 +156,24 @@ DownloadWrapper::receive_hash_done(ChunkHandle handle, const char* hash) {
   if (!info()->is_open())
     throw internal_error("DownloadWrapper::receive_hash_done(...) called but the download is not open.");
 
-  if (m_hashChecker->is_checking()) {
-    
+  if (m_hash_checker->is_checking()) {
     if (hash == NULL) {
-      m_hashChecker->receive_chunk_cleared(handle.index());
+      m_hash_checker->receive_chunk_cleared(handle.index());
 
     } else {
       if (std::memcmp(hash, chunk_hash(handle.index()), 20) == 0)
         m_main->file_list()->mark_completed(handle.index());
 
-      m_hashChecker->receive_chunkdone(handle.index());
+      m_hash_checker->receive_chunkdone(handle.index());
     }
 
-    m_main->chunk_list()->release(&handle, ChunkList::get_dont_log);
+    m_main->chunk_list()->release(&handle, ChunkList::release_dont_log);
     return;
   }
 
   // If hash == NULL we're clearing the queue, so do nothing.
   if (hash != NULL) {
-    if (!m_hashChecker->is_checked())
+    if (!m_hash_checker->is_checked())
       throw internal_error("DownloadWrapper::receive_hash_done(...) Was not expecting non-NULL hash.");
 
     // Receiving chunk hashes after stopping the torrent should be
@@ -192,14 +193,14 @@ DownloadWrapper::receive_hash_done(ChunkHandle handle, const char* hash) {
         finished_download();
 
       } else if (was_partial && data()->wanted_chunks() == 0) {
-        priority_queue_erase(&taskScheduler, &m_main->delay_partially_restarted());
-        priority_queue_update(&taskScheduler, &m_main->delay_partially_done(), cachedTime);
+        this_thread::scheduler()->erase(&m_main->delay_partially_restarted());
+        this_thread::scheduler()->update_wait_for(&m_main->delay_partially_done(), 0us);
       }
-    
-      if (!m_main->have_queue()->empty() && m_main->have_queue()->front().first >= cachedTime)
-        m_main->have_queue()->emplace_front(m_main->have_queue()->front().first + 1, handle.index());
+
+      if (!m_main->have_queue()->empty() && m_main->have_queue()->front().first >= this_thread::cached_time())
+        m_main->have_queue()->emplace_front(m_main->have_queue()->front().first + 1us, handle.index());
       else
-        m_main->have_queue()->emplace_front(cachedTime, handle.index());
+        m_main->have_queue()->emplace_front(this_thread::cached_time(), handle.index());
 
     } else {
       // This needs to ensure the chunk is still valid.
@@ -207,17 +208,24 @@ DownloadWrapper::receive_hash_done(ChunkHandle handle, const char* hash) {
     }
   }
 
-    data()->call_chunk_done(handle.object());
-    m_main->chunk_list()->release(&handle);
+  data()->call_chunk_done(handle.object());
+  m_main->chunk_list()->release(&handle, ChunkList::release_default);
 }
 
 void
-DownloadWrapper::check_chunk_hash(ChunkHandle handle) {
+DownloadWrapper::check_chunk_hash(ChunkHandle handle, bool hashing) {
   // TODO: Hack...
-  ChunkHandle new_handle = m_main->chunk_list()->get(handle.index(), ChunkList::get_blocking);
-  m_main->chunk_list()->release(&handle);
+  auto flags = ChunkList::get_blocking;
 
-  hash_queue()->push_back(new_handle, data(), std::bind(&DownloadWrapper::receive_hash_done, this, std::placeholders::_1, std::placeholders::_2));
+  if (hashing)
+    flags = flags | ChunkList::get_hashing;
+  else
+    flags = flags | ChunkList::get_not_hashing;
+
+  ChunkHandle new_handle = m_main->chunk_list()->get(handle.index(), flags);
+  m_main->chunk_list()->release(&handle, ChunkList::release_default);
+
+  hash_queue()->push_back(new_handle, data(), [this](auto c, auto h) { receive_hash_done(c, h); });
 }
 
 void
@@ -225,8 +233,8 @@ DownloadWrapper::receive_storage_error(const std::string& str) {
   m_main->stop();
   close();
 
-  m_main->tracker_controller()->disable();
-  m_main->tracker_controller()->close();
+  m_main->tracker_controller().disable();
+  m_main->tracker_controller().close();
 
   LT_LOG_STORAGE_ERRORS("%s", str.c_str());
 }
@@ -237,13 +245,13 @@ DownloadWrapper::receive_tracker_success(AddressList* l) {
   m_main->receive_connect_peers();
   m_main->receive_tracker_success();
 
-  utils::slot_list_call(info()->signal_tracker_success());
+  ::utils::slot_list_call(info()->signal_tracker_success());
   return inserted;
 }
 
 void
 DownloadWrapper::receive_tracker_failed(const std::string& msg) {
-  utils::slot_list_call(info()->signal_tracker_failed(), msg);
+  ::utils::slot_list_call(info()->signal_tracker_failed(), msg);
 }
 
 void
@@ -268,22 +276,25 @@ DownloadWrapper::receive_tick(uint32_t ticks) {
       } else if (info()->is_pex_active()) {
         info()->unset_flags(DownloadInfo::flag_pex_active);
 
-        for (ConnectionList::iterator itr = m_main->connection_list()->begin(); itr != m_main->connection_list()->end(); ++itr)
-          (*itr)->m_ptr()->set_peer_exchange(false);
+        for (auto& connection : *m_main->connection_list())
+          connection->m_ptr()->set_peer_exchange(false);
       }
     }
 
-    for (ConnectionList::iterator itr = m_main->connection_list()->begin(); itr != m_main->connection_list()->end(); )
+    for (auto itr = m_main->connection_list()->begin(); itr != m_main->connection_list()->end();)
       if (!(*itr)->m_ptr()->receive_keepalive())
         itr = m_main->connection_list()->erase(itr, ConnectionList::disconnect_available);
       else
         itr++;
   }
 
-  DownloadMain::have_queue_type* haveQueue = m_main->have_queue();
-  haveQueue->erase(std::find_if(haveQueue->rbegin(), haveQueue->rend(), [](auto& p) {
-    return (cachedTime - rak::timer::from_seconds(600)) < p.first;
-  }).base(), haveQueue->end());
+  auto have_queue = m_main->have_queue();
+
+  have_queue->erase(std::find_if(have_queue->rbegin(),
+                                 have_queue->rend(), [](auto& p) {
+                                     return this_thread::cached_time() - 600s < p.first;
+                                   }).base(),
+                    have_queue->end());
 
   m_main->receive_connect_peers();
 }
@@ -296,18 +307,18 @@ DownloadWrapper::receive_update_priorities() {
   data()->mutable_high_priority()->clear();
   data()->mutable_normal_priority()->clear();
 
-  for (FileList::iterator itr = m_main->file_list()->begin(); itr != m_main->file_list()->end(); ++itr) {
-    switch ((*itr)->priority()) {
+  for (auto& file : *m_main->file_list()) {
+    switch (file->priority()) {
     case PRIORITY_NORMAL:
     {
-      File::range_type range = (*itr)->range();
+      File::range_type range = file->range();
 
-      if ((*itr)->has_flags(File::flag_prioritize_first) && range.first != range.second) {
+      if (file->has_flags(File::flag_prioritize_first) && range.first != range.second) {
         data()->mutable_high_priority()->insert(range.first, range.first + 1);
         range.first++;
       }
 
-      if ((*itr)->has_flags(File::flag_prioritize_last) && range.first != range.second) {
+      if (file->has_flags(File::flag_prioritize_last) && range.first != range.second) {
         data()->mutable_high_priority()->insert(range.second - 1, range.second);
         range.second--;
       }
@@ -316,7 +327,7 @@ DownloadWrapper::receive_update_priorities() {
       break;
     }
     case PRIORITY_HIGH:
-      data()->mutable_high_priority()->insert((*itr)->range().first, (*itr)->range().second);
+      data()->mutable_high_priority()->insert(file->range().first, file->range().second);
       break;
     default:
       break;
@@ -335,13 +346,13 @@ DownloadWrapper::receive_update_priorities() {
   // The 'partially_done/restarted' signal only gets triggered when a
   // download is active and not completed.
   if (info()->is_active() && !file_list()->is_done() && was_partial != (data()->wanted_chunks() != 0)) {
-    priority_queue_erase(&taskScheduler, &m_main->delay_partially_done());
-    priority_queue_erase(&taskScheduler, &m_main->delay_partially_restarted());
-    
+    this_thread::scheduler()->erase(&m_main->delay_partially_done());
+    this_thread::scheduler()->erase(&m_main->delay_partially_restarted());
+
     if (was_partial)
-      priority_queue_insert(&taskScheduler, &m_main->delay_partially_done(), cachedTime);
+      this_thread::scheduler()->wait_for(&m_main->delay_partially_done(), 0us);
     else
-      priority_queue_insert(&taskScheduler, &m_main->delay_partially_restarted(), cachedTime);
+      this_thread::scheduler()->wait_for(&m_main->delay_partially_restarted(), 0us);
   }
 }
 
@@ -354,8 +365,7 @@ DownloadWrapper::finished_download() {
   // point.
   //
   // This needs to be seperated into a new function.
-  if (!m_main->delay_download_done().is_queued())
-    priority_queue_insert(&taskScheduler, &m_main->delay_download_done(), cachedTime);
+  this_thread::scheduler()->update_wait_for(&m_main->delay_download_done(), 0us);
 
   m_main->connection_list()->erase_seeders();
   info()->mutable_down_rate()->reset_rate();
